@@ -8,17 +8,19 @@
 #include "SnapBackend.h"
 #include "libdiscover_snap_debug.h"
 #include <KIO/ApplicationLauncherJob>
-#include <KService>
 #include <KLocalizedString>
+#include <KService>
 #include <QBuffer>
-#include <QDebug>
 #include <QDBusInterface>
+#include <QDebug>
 #include <QFile>
 #include <QFileInfo>
 #include <QImageReader>
 #include <QProcess>
+#include <QSettings>
 #include <QStandardItemModel>
 #include <Snapd/MarkdownParser>
+#include <Transaction/TransactionModel.h>
 
 #include <appstream/AppStreamUtils.h>
 #include <utils.h>
@@ -76,7 +78,7 @@ SnapResource::SnapResource(QSharedPointer<QSnapdSnap> snap, AbstractResource::St
     , m_installedSize(0)
     , m_downloadSize(0)
     , m_snap(snap)
-    , m_channel(QStringLiteral("latest/stable"))
+    , m_channel(m_snap->trackingChannel().isNull() ? u"latest/stable"_s : m_snap->trackingChannel())
 {
     setObjectName(snap->name());
 }
@@ -92,9 +94,9 @@ QString SnapResource::availableVersion() const
     return installedVersion();
 }
 
-QStringList SnapResource::categories()
+bool SnapResource::hasCategory(const QString &category) const
 {
-    return {QStringLiteral("Application")};
+    return category == QLatin1StringView("Application");
 }
 
 QString SnapResource::comment()
@@ -113,43 +115,46 @@ quint64 SnapResource::size()
 
 QVariant SnapResource::icon() const
 {
-    if (m_icon.isNull()) {
-        m_icon = [this]() -> QVariant {
-            const auto iconPath = m_snap->icon();
-            if (iconPath.isEmpty())
-                return QStringLiteral("package-x-generic");
+    if (m_state == AbstractResource::Installed) {
+        for (int i = 0; i < m_snap->appCount(); ++i) {
+            const auto app = m_snap->app(i);
 
-            if (!iconPath.startsWith(QLatin1Char('/')))
-                return QUrl(iconPath);
+            if (app->name() == m_snap->name()) {
+                QSettings desktopFile(app->desktopFile(), QSettings::IniFormat);
+                desktopFile.beginGroup("Desktop Entry");
+                const QString iconName = desktopFile.value("Icon").toString();
 
-            auto req = client()->getIcon(packageName());
-            connect(req, &QSnapdGetIconRequest::complete, this, &SnapResource::gotIcon);
-            req->runAsync();
-            return {};
-        }();
-    }
-    return m_icon;
-}
-
-void SnapResource::gotIcon()
-{
-    auto req = qobject_cast<QSnapdGetIconRequest *>(sender());
-    if (req->error()) {
-        qWarning() << "icon error" << req->errorString();
-        return;
+                if (!iconName.isEmpty()) {
+                    return iconName;
+                }
+            }
+        }
     }
 
-    auto icon = req->icon();
+    if (!m_snap->icon().isEmpty() && !m_snap->icon().startsWith(QLatin1Char('/'))) {
+        return QUrl(m_snap->icon());
+    }
+
+    for (int m = 0; m < m_snap->mediaCount(); ++m) {
+        if (m_snap->media(m)->type() == QStringLiteral("icon")) {
+            return QUrl(m_snap->media(m)->url());
+        }
+    }
+
+    QSnapdClient client;
+    auto req = client.getIcon(m_snap->name());
+    req->runSync();
+
+    if (req->error() != QSnapdRequest::NoError) {
+        return QStringLiteral("package-x-generic");
+    }
 
     QBuffer buffer;
-    buffer.setData(icon->data());
+    buffer.setData(req->icon()->data());
     QImageReader reader(&buffer);
+    const auto theIcon = QVariant::fromValue<QImage>(reader.read());
 
-    auto theIcon = QVariant::fromValue<QImage>(reader.read());
-    if (theIcon != m_icon) {
-        m_icon = theIcon;
-        Q_EMIT iconChanged();
-    }
+    return theIcon.isNull() ? QStringLiteral("package-x-generic") : theIcon;
 }
 
 QString SnapResource::installedVersion() const
@@ -468,21 +473,12 @@ QStringList SnapResource::topObjects() const
 
 QString SnapResource::channel()
 {
-    if (isInstalled()) {
-        auto req = client()->getSnap(packageName());
-        req->runSync();
-        return req->error() ? QString() : req->snap()->trackingChannel();
-    }
     return m_channel;
 }
 
 QString SnapResource::author() const
 {
-    QString author = m_snap->publisherDisplayName();
-    if (m_snap->publisherValidation() == QSnapdEnums::PublisherValidationVerified) {
-        author += QStringLiteral(" ✅");
-    }
-    return author;
+    return m_snap->publisherDisplayName();
 }
 
 void SnapResource::setChannel(const QString &channelName)
@@ -493,16 +489,10 @@ void SnapResource::setChannel(const QString &channelName)
         return;
     }
     Q_ASSERT(isInstalled());
-    auto request = client()->switchChannel(m_snap->name(), channelName);
-
-    const auto currentChannel = channel();
-    request->runAsync();
-    connect(request, &QSnapdRequest::complete, this, [this, currentChannel]() {
-        const auto newChannel = channel();
-        if (newChannel != currentChannel) {
-            Q_EMIT channelChanged(newChannel);
-        }
-    });
+    m_channel = channelName;
+    Q_EMIT channelChanged(channelName);
+    setState(AbstractResource::Upgradeable);
+    TransactionModel::global()->addTransaction(backend()->installApplication(this));
 }
 
 quint64 SnapResource::installedSize() const
@@ -526,6 +516,19 @@ void SnapResource::updateSizes()
     }
 }
 
+// Snap Links
+
+QUrl SnapResource::homepage()
+{
+    return QUrl(m_snap->website());
+}
+
+QUrl SnapResource::url() const
+{
+    return QUrl(QStringLiteral("https://snapcraft.io/%1").arg(m_snap->name()));
+
+}
+
 void SnapResource::refreshSnap()
 {
     auto request = client()->find(QSnapdClient::FindFlag::MatchName, m_snap->name());
@@ -596,5 +599,29 @@ QObject *SnapResource::channels(QObject *parent)
     return new Channels(this, parent);
 }
 
+QString SnapResource::verifiedMessage() const
+{
+    switch (m_snap->publisherValidation()) {
+    case QSnapdEnums::PublisherValidationVerified:
+        return i18n("Verified Publisher on snapcraft.io");
+    case QSnapdEnums::PublisherValidationStarred:
+        return i18n("Starred Publisher on snapcraft.io");
+    default:
+        return {};
+    }
+}
+
+QString SnapResource::verifiedIconName() const
+{
+    switch (m_snap->publisherValidation()) {
+    case QSnapdEnums::PublisherValidationVerified:
+        return QStringLiteral("checkmark");
+    case QSnapdEnums::PublisherValidationStarred:
+        return QStringLiteral("starred-symbolic");
+    default:
+        return {};
+    }
+}
+
 #include "SnapResource.moc"
 #include "moc_SnapResource.cpp"
