@@ -73,7 +73,12 @@ int main(int argc, char **argv)
     }
 
     KDBusService service(KDBusService::Unique | startup);
-    notifier.setVisible(!hide);
+
+    // Otherwise the QEventLoopLocker in KStatusNotifierItem quits the
+    // application when the status notifier is destroyed
+    app.setQuitLockEnabled(false);
+
+    notifier.setStatusNotifierEnabled(!hide);
 
     return app.exec();
 }
